#include "ieee_fun_types.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define MILLION 1000000

unsigned  getbits( unsigned x, unsigned p, unsigned n) 
{
    /* get n bits from position p */
	return  (( x >> (p+1-n)) & ~(~0 << n));
}

/* use below to get rid of warning message */
void usleep(unsigned long usec);


int main()
{
  int lam=0;
  int ext;
  int branch=18;
  int crate=4;
  int slot,subad,function,temp,qres;

  int return_status = CAM_S_SUCCESS;
  unsigned short dataw[2]={0,0};

  int i=0;
  int adc0,adc1,adc2,ad114;

 	
  /***************** OPEN DATA FILE ********************/
  
  /* File to save data */
  FILE *data_file;
  char file[20] = ("data.txt");
  char ctemp[4];
  int r;
  int run;
  
  run = 0;	/* force run 000 */
  printf("run = %d\n", run);
  sprintf( &ctemp[0], "%03i", run);
  for (r = 0; r<3; ++r) file[r+9]=ctemp[r];
  printf("file name = %s\n", file); 
  
  /* Truncate to zero length or create file for writing ascii data output*/
  data_file = fopen(file,"w");
  if ( data_file == NULL ) {
    printf("Failed to open an output file to write data!\n");
    return 2;}
  
    fprintf(data_file,"i adc0 adc1 adc2 ad114\n");
  
  /****************** OPEN Jorway CAMAC interface **********************/

  printf ("\n Opening Jorway411 branch highway");
  
  if ((return_status = cdchn(branch, 1, 0)) !=  CAM_S_SUCCESS)  {
    printf("Error opening branch=%d\n",branch);
    exit(1);
  };
  
  ccctype(branch,0,1);  /* parallel - Jor 73A is considered parallel */
  
  /**************************** Initialize CAMAC **************************/
  
  printf("\n Initializing CAMAC\n");
  slot=2; subad=0; 
  
  /* encode the branch, crate, slot, 
     and subad arguments into ext */
  cdreg(&ext,branch,crate,slot,subad);
  
  /* crate clear */
  cccc(ext);
  /* crate initialize */
  cccz(ext);
  /* remove crate inhibit */
  ccci(ext,0);

  function=0;


  /****************** SET UP 1st Ortec AD411 ADC ***********************/
  /*	0x5F00 = 1001 1111 0000 0000

	B16 =1 => Overflow-suppresion enable when 0
	B15  1	  LAM enable
	B14  1    Notused
	B13 =1 => Singles mode
	---
	B12 =1 => Enable Master Gate when 0, Master Gate ignored when 1
	B11 =1 => Enable Local Gate when 0, ignor local gate when 1
	B10 =1 => ECL port enable : 0=ECL prot readout enabled, 1 Camac readout enabled
	B9  =1 => Zero-suppression enable : when 0 ADC with zeros for are skipped during readout
	---
	b1 to b8 => Virt station number
   */
 
  printf("\n Initializing AD114");

  slot=2; 

  /* Clear module */

  subad=0; 
  cdreg(&ext,branch,crate,slot,subad);
  cssa(9,ext,dataw,&qres);
  printf(" (using slot=%d) ",slot);

  /* Write to control register and read back  */
  
  dataw[0]=0xFF00;    
  
  cssa(16,ext,dataw,&qres);  /* Using FERA */
  
  cssa(0,ext,&dataw[0],&qres);  
  printf("\n AD114 Control Register: 0x%x", dataw[0]);	
  
  dataw[0]=0x00D7;
  printf("\n Write %X to LLD", dataw[0]);
  subad=0;
  function=17;
  cdreg(&ext,branch,crate,slot,subad);
  cssa(function,ext,&dataw[0],&qres);
  
  /* Read lower level discriminator setting */
  cssa(1,ext,&dataw[0],&qres);  
  printf("\n AD114 LLD: 0x%x", temp & 0xFF);	
  
  /* Read upper level discriminator setting */
  subad=1;
  cdreg(&ext,branch,crate,slot,subad);
  cssa(1,ext,&dataw[0],&qres);  
  printf("\n AD114 ULD: 0x%x", dataw[0] & 0xFF);	
  printf("\n");
  
  /* end of enable AD114 */
  

  
  printf("\n Initializing 2249A");
 
 /* setup lecroy 2249 */
	slot = 3;
	subad=0;
	cdreg(&ext,branch,crate,slot,subad);
	cssa(26,ext,dataw,&qres);
  
  

  /***************************************************************/
  /************************ START EVENT LOOP *********************/
  /***************************************************************/

  fprintf(stderr, "\nStarting event loop\n");
  
  for(i=0; i<10000; i++) {
    
    
    /* Clear 2249 module */

    slot=3;
    subad=0; 
    cdreg(&ext,branch,crate,slot,subad);
    cssa(9,ext,dataw,&qres);
    cssa(26,ext,dataw,&qres);
    
    lam = 0;	/* get ready for next test */
    
    while ( lam == 0 ) {
      
      slot=3;subad=0;function=8;	/* test lam status */
      cdreg(&ext,branch,crate,slot,subad);
      cssa(function,ext,dataw,&qres);
      lam = qres;
      if ( lam == 1 ) break;
      usleep(10000);	/* otherwise sleep for 10 milliseconds */
    }

    /* read out 2249 */
    
    slot=3;subad=0;    /* read out channel 0 */	
    cdreg(&ext,branch,crate,slot,subad);
    cssa(0,ext,dataw,&qres);    
    printf(" \n 2249 channel 0 word: %d",dataw[0]);
    adc0=dataw[0];

    subad=1;    /* read out channel 1 */	
    cdreg(&ext,branch,crate,slot,subad);
    cssa(0,ext,dataw,&qres);    
    printf(" \n 2249 channel 1 word: %d",dataw[0]);
    adc1=dataw[0];

    subad=2;    /* read out channel 2 */	
    cdreg(&ext,branch,crate,slot,subad);
    cssa(0,ext,dataw,&qres);    
    printf(" \n 2249 channel 2 word: %d",dataw[0]);
    adc2=dataw[0];

    subad=0; /* clear event and LAM */
    cdreg(&ext,branch,crate,slot,subad);
    cssa(9,ext,dataw,&qres);

    /*  sleep for 10 milliseconds for AD114 conversion  */

    usleep(10000);
  
    /* read out AD114 */

    slot=2;subad=0;    
    cdreg(&ext,branch,crate,slot,subad);
    cssa(2,ext,dataw,&qres);    
    printf(" \n AD114 word: %d",dataw[0]);
    ad114=dataw[0];
  
    /* write to data file */
    printf("\n %d %d %d %d %d\n",i,adc0,adc1,adc2,ad114);
    fprintf(data_file,"%d %d %d %d %d\n",i,adc0,adc1,adc2,ad114);

  } /* end of for */
  


  /***************************************************************/
  /************************ END EVENT LOOP *********************/
  /***************************************************************/
  
  printf("\n\n***** End of run *****\n\n");
  
  return 0;
}
